from typing import Set

import pytest

from model.http.info.parameter.ParameterModel import Parameters, Parameter, AbstractParameter


def test_should_have_duplicate_parameters() -> None:
    parameters: Parameters = Parameters.from_list([
        Parameter.from_key_value('a', 'b'),
        Parameter.from_key_value('c', 'd'),
        Parameter.from_key_value('a', 'b'),
    ])
    assert len(parameters.get_parameters_set()) == 2

    expected: Set[AbstractParameter] = {
        Parameter.from_key_value('a', 'b'),
        Parameter.from_key_value('c', 'd')
    }

    assert all(
        parameters in expected for parameters in parameters.get_parameters_set())



from http import HTTPStatus

from view.item.LabelItem import LabelItem


class StatusCodeWrapper(LabelItem):
    DEFAULT_CSS = """
               StatusCodeWrapper {
                   height: auto;
                   margin: 1 0;
                   border-title-align: left;
                   padding: 1 2;
               }

               StatusCodeWrapper .label {
                   text-style: bold; 
                   color: $text-muted;
               }

               StatusCodeWrapper .value {
                   color: $text;
               }

               StatusCodeWrapper.success {
                   border: round green;
               }

               StatusCodeWrapper.success .label {
                   color: green;
               }

               StatusCodeWrapper.error {
                   border: round red;
               }

               StatusCodeWrapper.error .label {
                   color: red;
               }
               """

    def __init__(self, value: HTTPStatus, **kwargs):
        super().__init__(label="Status Code",
                         value=f"Code:{value.value} → {value.name}", **kwargs)
        self.remove_class("success", "error")
        self.add_class("success" if value.value < 400 else "error")



import re
from abc import ABC, abstractmethod
from re import Pattern, Match
from typing import TypeVar, Generic


from model.builder.BuilderModel import AbstractBuilder
from model.cli.command.FlagFactory import AbstractFlagFactory
from model.cli.command.FlagModel import AbstractFlag

_I = TypeVar("_I")
_T = TypeVar("_T")


class AbstractExtractCommandFlagStrategy(ABC, Generic[_I, _T]):
    _my_query_data_builder: AbstractBuilder[_T]
    _my_flag_factory: AbstractFlagFactory

    def __init__(self, my_query_data_builder: AbstractBuilder[_T],
                 flag_factory: AbstractFlagFactory):
        self._my_query_data_builder = my_query_data_builder
        self._my_flag_factory = flag_factory

    @abstractmethod
    def extract_flag_representation(self, input_command: _I,
                                    ) -> set[AbstractFlag]:
        pass


class StringCommandFlagStrategy(AbstractExtractCommandFlagStrategy[str, _T]):
    _my_flag_with_value_regex: Pattern[str] = re.compile(
        r'-(\w+)(?:=(.+?))?(?=\s+-\w|$)', re.DOTALL)

    def __init__(self, my_query_data_builder: AbstractBuilder[_T],
                 flag_factory: AbstractFlagFactory):
        super().__init__(my_query_data_builder, flag_factory)

    def extract_flag_representation(self, input_command: str,
                                    ) -> set[AbstractFlag]:
        rest: str = input_command.split(
            None, 1)[1] if ' ' in input_command else ''
        all_flags: set[AbstractFlag] = set()
        for matches in self._my_flag_with_value_regex.finditer(rest):
            if matches:
                flag_value: str = matches.group(2) if matches.group(2) else ''
                all_flags.add(self._my_flag_factory.create_flag(
                    matches.group(1), flag_value, self._my_query_data_builder))
        return all_flags



from abc import ABC

from model.builder.BuilderModel import AbstractBuilder
from model.cli.command.FlagModel import AbstractFlag, AbstractFlagValue


def create_flag(flag_class: type[AbstractFlag],
                flag_value_class: type[AbstractFlagValue],
                flag_value: str,
                builder: AbstractBuilder) -> AbstractFlag:
    return flag_class(my_flag_value=flag_value_class(my_query_builder=builder, flag_value=flag_value))


class AbstractFlagFactory(ABC):
    _my_flags: dict[str, tuple[type[AbstractFlag], type[AbstractFlagValue]]]

    def create_flag(self, flag_name: str,
                    flag_value: str,
                    builder: AbstractBuilder) -> AbstractFlag:
        if flag_name not in self._my_flags:
            raise KeyError(
                f"{flag_name} not exist, specify the correct flag name.")
        return create_flag(flag_class=self._my_flags[flag_name][0],
                           flag_value_class=self._my_flags[flag_name][1],
                           flag_value=flag_value,
                           builder=builder)



from http import HTTPMethod, HTTPStatus

from model.http.info.body.BodyModel import Body
from model.http.info.content.RequestModel import RequestContent, RequestContentBuilder
from model.http.info.data.DataModel import AbstractHttpData, HttpData
from model.http.info.endpoint.EndpointModel import BasicEndpoint
from model.http.info.header.HeaderModel import Headers, Header, AbstractHeader
from model.http.info.parameter.ParameterModel import Parameters, Parameter, AbstractParameter


def test_should_be_possible_to_retrieve_data_attributes():
    method: HTTPMethod = HTTPMethod.GET
    request: RequestContent = (RequestContentBuilder()
                               .add_body(Body.from_string('random body'))
                               .add_headers(
        Headers.from_list([
            Header.from_tuple(('key', 'value'))
        ]))
        .add_parameters(Parameters.from_list([Parameter.from_key_value('key', 'value')]))
        .add_endpoint(BasicEndpoint.create_endpoint('localhost', 80))
        .build()
    )
    data: AbstractHttpData = HttpData.create_data(method,
                                                  request)
    expected_parameters: set[AbstractParameter] = {
        Parameter.from_key_value('key', 'value')}
    expected_headers: set[AbstractHeader] = {
        Header.from_tuple(('key', 'value'))}
    assert data.get_http_method() == method
    assert data.get_http_request() is not None
    assert all(param in expected_parameters for param in data.get_http_request(
    ).get_parameters().get_parameters_set())
    assert all(header in expected_headers for header in data.get_http_request(
    ).get_headers().get_headers_set())
    assert data.get_http_request().get_endpoint().dump() == "localhost:80"



from abc import abstractmethod
from http import HTTPMethod

from model.http.engine.EngineModel import AbstractEngine
from model.http.info.data.DataModel import AbstractHttpData
from model.http.info.content.ResponseModel import ResponseContent


class AbstractHttpEngine(AbstractEngine[ResponseContent]):
    """
    This is the engine for making all the http requests
    """
    _my_engine_name: str

    def __init__(self, my_engine_name: str):
        self._my_engine_name = my_engine_name

    def get_engine_name(self) -> str:
        return self._my_engine_name

    def analyze(self, request: AbstractHttpData) -> ResponseContent:
        return self.do_query(request)

    def do_query(self, http_request: AbstractHttpData) -> ResponseContent:
        match http_request.get_http_method():
            case HTTPMethod.GET:
                return self._do_get(http_request)
            case HTTPMethod.POST:
                return self._do_post(http_request)
            case HTTPMethod.PUT:
                return self._do_put(http_request)
            case HTTPMethod.DELETE:
                return self._do_delete(http_request)
            case HTTPMethod.PATCH:
                return self._do_patch(http_request)
            case not_supported:
                raise ValueError(
                    f"The input http method: {not_supported} is not supported.")

    @abstractmethod
    def _do_get(self, http_request: AbstractHttpData) -> ResponseContent:
        pass

    @abstractmethod
    def _do_post(self, http_request: AbstractHttpData) -> ResponseContent:
        pass

    @abstractmethod
    def _do_put(self, http_request: AbstractHttpData) -> ResponseContent:
        pass

    @abstractmethod
    def _do_delete(self, http_request: AbstractHttpData) -> ResponseContent:
        pass

    @abstractmethod
    def _do_patch(self, http_request: AbstractHttpData) -> ResponseContent:
        pass



from abc import ABC, abstractmethod
from typing import TypeVar, Generic

from model.cli.command.CommandRepresentation import AbstractExtractCommandFlagStrategy
from model.cli.command.FlagModel import AbstractFlag
from model.cli.command.OutputModel import AbstractOutput

_T = TypeVar("_T", bound=AbstractOutput)


class AbstractCommand(ABC, Generic[_T]):
    _my_command_name: str
    _my_flag_strategy: AbstractExtractCommandFlagStrategy

    def __init__(self, command_name: str, flag_strategy: AbstractExtractCommandFlagStrategy):
        self._my_command_name = command_name
        self._my_flag_strategy = flag_strategy

    @abstractmethod
    def execute_command(self, command: str) -> _T:
        """
        This method extracts from the input command all the flags, execute for each flag the logic and then it returns the output of the command
        :param command: input flag to parse
        :return: see above.
        """
        pass

    @abstractmethod
    def prepare_flags(self, command: str) -> set[AbstractFlag]:
        pass

    def get_description(self) -> str:
        """
        This method returns the description of the command.
        :return:
        """
        return "The following command supports all these flags:"



from model.http.engine.HttpEngineModel import AbstractHttpEngine
from model.http.engine.StandardHttpEngineModel import StandardHttpEngine, STANDARD_ENGINE_NAME

standard_engine: AbstractHttpEngine = StandardHttpEngine()

all_engines: dict[str, AbstractHttpEngine] = {
    STANDARD_ENGINE_NAME: standard_engine
}


def get_engine(engine_name: str = '') -> AbstractHttpEngine:
    if engine_name not in all_engines:
        raise ValueError('The engine name is not supported!')
    return all_engines[engine_name]



from abc import ABC, abstractmethod
from typing import TypeVar, Generic

from model.http.info.data.DataModel import AbstractHttpData

_B = TypeVar("_B")


class AbstractEngine(ABC, Generic[_B]):
    @abstractmethod
    def analyze(self, request: AbstractHttpData) -> _B:
        pass



from abc import ABC
from http import HTTPStatus
from typing import Iterable, TypeVar

from model.http.info.endpoint.EndpointModel import AbstractEndpoint
from model.http.info.body.BodyModel import AbstractBody
from model.http.info.header.HeaderModel import Headers
from model.http.info.parameter.ParameterModel import Parameters

_SENTINEL = object()

_T = TypeVar('_T', bound=Iterable)


class AbstractContent(ABC):
    """
    This class represents an http content.
    It has an endpoint, a status code, a collection of headers and a body.
    """
    _my_endpoint: AbstractEndpoint
    _my_status_code: HTTPStatus
    _my_headers: Headers
    _my_parameters: Parameters
    _my_body: AbstractBody

    def __init__(self, endpoint: AbstractEndpoint, headers: Headers, parameters: Parameters, body: AbstractBody):
        self._my_endpoint = endpoint
        self._my_headers = headers
        self._my_parameters = parameters
        self._my_body = body

    def get_endpoint(self) -> AbstractEndpoint:
        """
        This method returns the endpoint.
        :return:
        """
        return self._my_endpoint

    def get_status_code(self) -> HTTPStatus:
        """
        This method returns the current http status code.
        :return: see above
        """
        return self._my_status_code

    def get_headers(self) -> Headers:
        """
        This method returns the collection of headers.
        :return: see above
        """
        return self._my_headers

    def get_parameters(self) -> Parameters:
        """
        This method returns all the parameters.
        :return:
        """
        return self._my_parameters

    def get_body(self) -> AbstractBody:
        """
        This method returns the body.
        :return: see above
        """
        return self._my_body



from textual.app import ComposeResult
from textual.containers import Vertical
from textual.widgets import Static


class LabelItem(Vertical, Static):

    _my_label: str
    _my_value: str

    def __init__(self, label: str, value: str, **kwargs):
        super().__init__(value, **kwargs)
        self._my_label = label
        self._my_value = value

    def compose(self) -> ComposeResult:
        yield Static(self._my_label, classes="label")
        yield Static(self._my_value, classes="value")



from abc import ABC, abstractmethod
from typing import Generic, TypeVar

_T = TypeVar('_T')


class IterableContent(ABC, Generic[_T]):
    """
    Rerpesents a content that can be iterated over.
    """
    @abstractmethod
    def dump(self) -> _T:
        """
        Translate the input collection into another type.
        :return: see above.
        """
        pass



from abc import abstractmethod

from model.cli.command.CommandModel import AbstractCommand
from model.cli.command.CommandRepresentation import StringCommandFlagStrategy
from model.cli.command.FlagModel import AbstractFlag
from model.cli.command.http.Contants import HTTP_COMMAND_NAME
from model.cli.command.http.HttpOutputModel import HttpOutput
from model.cli.command.http.flags.HttpFlagsFactory import HttpFlagFactory
from model.http.engine.HttpEngineModel import AbstractHttpEngine
from model.http.info.data.DataModel import AbstractHttpData, HttpDataBuilder
from model.http.info.content.ResponseModel import ResponseContent


class AbstractHttpCommand(AbstractCommand[HttpOutput]):
    """
    This class represent the definition of an http command, it works with the HttpOutput class.

    An example of this class from cli is: http ...
    In order to work this class uses the HttpDataBuilder class for creating the input data for the query and it returns
    output wrapped in the HttpOutput class.
    """
    _my_http_engine: AbstractHttpEngine
    _my_http_data_builder: HttpDataBuilder

    def __init__(self, http_engine: AbstractHttpEngine, command_name: str):
        self._my_http_data_builder = HttpDataBuilder()
        self._my_http_engine = http_engine
        self._my_http_command_name = command_name

        super().__init__(command_name, StringCommandFlagStrategy(
            self._my_http_data_builder, HttpFlagFactory()))

    @abstractmethod
    def execute_command(self, command: str) -> HttpOutput:
        pass


class HttpCommand(AbstractHttpCommand):
    """
    Actual implementation of the Http command class.
    """

    def __init__(self, http_engine: AbstractHttpEngine):
        super().__init__(http_engine,
                         HTTP_COMMAND_NAME)

    def execute_command(self, command: str) -> HttpOutput:
        flags: set[AbstractFlag] = self.prepare_flags(command)
        for flag in flags:
            # given the internal value it works with the builder
            flag.get_flag_value().match_value()
        return self._execute_http_command(self._my_http_data_builder.build())

    def prepare_flags(self, command: str) -> set[AbstractFlag]:
        """
        This method allows to extract end create all the defined flags in the input command.
        :param command: input command where all the flags are specified.
        :return: the set of all defined flags.
        """
        return self._my_flag_strategy.extract_flag_representation(command)

    def get_description(self) -> str:
        return f"Command name: {self._my_command_name}. \n" + super().get_description()

    def _execute_http_command(self, request: AbstractHttpData) -> HttpOutput:
        """
        This method allows to execute the http command.
        :param request: input request created from the input flags.
        :return: the output of the http command.
        """
        response: ResponseContent = self._my_http_engine.do_query(request)
        return HttpOutput(response)




from model.builder.BuilderModel import AbstractBuilder
from model.http.info.body.BodyModel import AbstractBody
from model.http.info.content.ContentModel import AbstractContent
from model.http.info.endpoint.EndpointModel import AbstractEndpoint
from model.http.info.header.HeaderModel import Headers
from model.http.info.parameter.ParameterModel import Parameters

_SENTINEL = object()


class RequestContent(AbstractContent):
    """
    This class represents a http Request.
    """

    def __init__(self, _sentinel: object = None, *, endpoint: AbstractEndpoint, headers: Headers, parameters: Parameters, body: AbstractBody):
        if _sentinel is None:
            raise TypeError(
                "In order to create a Request you have to use the factory http.")
        super().__init__(endpoint, headers, parameters, body)

    @classmethod
    def create_request(cls, endpoint: AbstractEndpoint, headers: Headers, parameters: Parameters, body: AbstractBody) -> 'RequestContent':
        return cls(_sentinel=_SENTINEL, endpoint=endpoint, headers=headers, parameters=parameters, body=body)


class RequestContentBuilder(AbstractBuilder[RequestContent]):
    _my_endpoint: AbstractEndpoint
    _my_headers: Headers
    _my_parameters: Parameters
    _my_body: AbstractBody

    def add_endpoint(self, endpoint: AbstractEndpoint) -> 'RequestContentBuilder':
        self._my_endpoint = endpoint
        return self

    def add_headers(self, headers: Headers) -> 'RequestContentBuilder':
        self._my_headers = headers
        return self

    def add_parameters(self, parameters: Parameters) -> 'RequestContentBuilder':
        self._my_parameters = parameters
        return self

    def add_body(self, body: AbstractBody) -> 'RequestContentBuilder':
        self._my_body = body
        return self

    def build(self) -> RequestContent:
        if self._my_endpoint and self._my_headers and self._my_parameters and self._my_body:
            return RequestContent.create_request(endpoint=self._my_endpoint,
                                                 headers=self._my_headers,
                                                 parameters=self._my_parameters,
                                                 body=self._my_body)
        raise TypeError(
            "In order to build the Request all the parameters has to be set.")



from model.cli.command.OutputModel import AbstractOutput
from model.http.info.content.ResponseModel import ResponseContent


class HttpOutput(AbstractOutput[ResponseContent]):
    def __init__(self, output: ResponseContent):
        super().__init__(output)

    def wrap_output(self) -> str:
        output: str = f"Status Code: {self.get_output().get_status_code()} - {self.get_output().get_status_code().name}\n"
        output += f"Headers:\n[ {self.get_output().get_headers()}]\n"
        output += f"Body:\n{self.get_output().get_body().get_content()}\n"
        return output



from typing import Any

from model.http.info.body.BodyModel import AbstractBody, Body


def test_should_be_possible_to_create_empty_body():
    body: AbstractBody = Body.from_string()
    assert len(body.get_content()) == 0


def test_body_should_be_possible_to_retrieve_it():
    body: AbstractBody = Body.from_string('Random Body')
    assert body.get_content() is not None
    assert body.get_content() == 'Random Body'


def test_should_be_possible_to_create_body_from_empty_dict():
    body: AbstractBody = Body.from_dict({})
    assert body.get_content() is not None
    assert body.get_content() == ''


def test_should_be_possible_to_create_body_from_dict():
    content: dict[Any, Any] = {
        'key': 'value',
        'key2': [
            'a',
            'b'
        ],
        'key3': {
            'key4': [1]
        }
    }
    import json
    body: AbstractBody = Body.from_dict(content)
    assert body.get_content() is not None
    assert body.get_content() == json.dumps(content)



from http import HTTPStatus

import requests
from requests import Response

from model.cli.command.http.Contants import STANDARD_ENGINE_NAME
from model.http.engine.HttpEngineModel import AbstractHttpEngine
from model.http.info.body.BodyModel import Body
from model.http.info.header.HeaderModel import Headers
from model.http.info.data.DataModel import AbstractHttpData
from model.http.info.parameter.ParameterModel import Parameters
from model.http.info.content.ResponseModel import ResponseContent


class StandardHttpEngine(AbstractHttpEngine):
    def __init__(self):
        super().__init__(STANDARD_ENGINE_NAME)

    def _do_get(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(),
                                     http_request)

    def _do_post(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(),
                                     http_request)

    def _do_put(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(),
                                     http_request)

    def _do_delete(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(), http_request)

    def _do_patch(self, http_request: AbstractHttpData) -> ResponseContent:
        return self._execute_request(http_request.get_http_method(), http_request)

    def _execute_request(self, method: str, request: AbstractHttpData) -> ResponseContent:
        response: Response = requests.request(method=method,
                                              url=request.get_http_request().get_endpoint().dump(),
                                              params=request.get_http_request().get_parameters().dump(),
                                              headers=request.get_http_request().get_headers().dump(),
                                              data=request.get_http_request().get_body().get_content())
        return ResponseContent.create_response(endpoint=request.get_http_request().get_endpoint(),
                                               status_code=HTTPStatus(
                                                   response.status_code),
                                               headers=Headers.from_dictionary(
                                                   response.headers),
                                               parameters=Parameters.from_list(
                                                   []),
                                               body=Body.from_string(response.text))



from view.item.StringViewerItem import StringViewerItem
import json


class BodyWrapper(StringViewerItem):
    DEFAULT_CSS = """
    BodyWrapper {
        margin: 1 0;
    }

    BodyWrapper ScrollableContainer {
        border: round #30363d;
        background: #161b22;
        padding: 1 2;
        height: 16;
        scrollbar-color: #30363d;
        scrollbar-background: #0d1117;
        scrollbar-size-vertical: 1;
    }
    """

    def __init__(self, content: str, title: str = "Body", **kwargs):
        try:
            super().__init__(json.loads(content), title, **kwargs)
        except json.decoder.JSONDecodeError:
            super().__init__(content, title, **kwargs)



from typing import TypeVar, Generic

from textual.binding import Binding
from textual.containers import ScrollableContainer
from textual.widgets import Pretty

T = TypeVar("T")


class CustomScrollableContainer(ScrollableContainer, Generic[T]):
    BINDINGS = [Binding("w", "up", "Scroll Up"),
                Binding("s", "down", "Scroll Down")]

    def __init__(self, content: T):
        super().__init__(Pretty(content))

    def action_down(self):
        self.scroll_down()

    def action_up(self):
        self.scroll_up()



from http import HTTPStatus
from typing import cast

from model.cli.command.http.HttpOutputModel import HttpOutput
from model.parser.ParserModel import ParserFactory
from tests.util.BaseServerTestCase import BaseServerTestCase


class ParseTest(BaseServerTestCase):
    __test__ = True

    def __init__(self, method_name: str = "runTest"):
        super().__init__('http://localhost', 8080, method_name)

    def test_should_be_possible_to_run_http_command(self) -> None:
        command: str = "http -method=get -endpoint=http://localhost:8080"
        output: HttpOutput = ParserFactory[str]().from_command(
            input_command=command).execute_command(command)
        assert output.get_output().get_status_code() == HTTPStatus.OK



from http import HTTPStatus

from model.http.info.body.BodyModel import AbstractBody, Body
from model.http.info.content.ResponseModel import ResponseContent
from model.http.info.endpoint.EndpointModel import AbstractEndpoint, BasicEndpoint
from model.http.info.header.HeaderModel import Headers
from model.http.info.parameter.ParameterModel import Parameters, Parameter
from tests.model.http.info.content.BaseContentTest import BaseContentTestCase


class TestResponse(BaseContentTestCase):
    __test__ = True
    _my_endpoint: AbstractEndpoint
    _my_headers: Headers
    _my_parameters: Parameters
    _my_body: AbstractBody
    _my_status_code: HTTPStatus

    def __init__(self, method: str = "runTest"):
        self._my_endpoint: AbstractEndpoint = BasicEndpoint.create_endpoint(
            'localhost', 8080)
        self._my_headers: Headers = Headers.from_dictionary(
            {'content-type': 'application/json'})
        self._my_parameters: Parameters = Parameters.from_list(
            [Parameter.from_tuple(('key', 'value'))])
        self._my_body: AbstractBody = Body.from_string("random")
        self._my_status_code = HTTPStatus.OK

        super().__init__(ResponseContent.create_response(endpoint=self._my_endpoint,
                                                         status_code=self._my_status_code,
                                                         headers=self._my_headers,
                                                         parameters=self._my_parameters,
                                                         body=self._my_body
                                                         ), method)

    def test_status_code(self):
        assert self._my_status_code == HTTPStatus.OK



from abc import ABC, abstractmethod
from re import Pattern
from typing import Generic, NoReturn

from typing_extensions import TypeVar

from model.builder.BuilderModel import AbstractBuilder

_T = TypeVar("_T", bound=AbstractBuilder)


class AbstractFlagValue(ABC, Generic[_T]):
    """
    This class represent the value of a flag, so what is after the equal in the flag definition.
    -myflag=value <- this
    """
    _my_query_builder: _T
    _my_representation: Pattern
    _my_flag_value: str

    def __init__(self, my_query_builder: _T, flag_value: str):
        self._my_query_builder = my_query_builder
        self._my_flag_value = flag_value

    def is_valid_flag_value(self, flag_value: str) -> bool:
        """
        This method allows to check if the flag is valid.
        :param flag_value: input string that represent the flag value
        :return: see above
        """
        return self._my_representation.match(flag_value) is not None

    @abstractmethod
    def match_value(self) -> None:
        """
        This method takes from the internal flag value the flag value specified by the user and does something
        with the internal builder.
        :return: see above.
        """
        pass


class AbstractFlag(ABC, Generic[_T]):
    """
    This class represents a specific command flag, defined in this way:
    -name=value
    """
    _my_name: str
    _my_flag_value: AbstractFlagValue[_T]

    def __init__(self, my_flag_value: AbstractFlagValue[_T]):
        self._my_flag_value = my_flag_value

    @abstractmethod
    def get_flag_descritpion(self) -> str:
        pass

    def get_flag_value(self) -> AbstractFlagValue[_T]:
        return self._my_flag_value

    def __repr__(self) -> str:
        return f"{self._my_name}: {self.get_flag_descritpion()}"



STANDARD_ENGINE_NAME: str = 'requests'
# FLAGS
HTTP_FLAG_METHOD: str = "method"
HTTP_FLAG_ENDPOINT: str = "endpoint"
HTTP_FLAG_HEADERS: str = "headers"
HTTP_FLAG_PARAMETERS: str = "parameters"
HTTP_FLAG_BODY: str = "body"

# HTTP COMMAND NAME
HTTP_COMMAND_NAME = "http"



from view.item.LabelItem import LabelItem


class EndpointWrapper(LabelItem):
    DEFAULT_CSS = """
    EndpointWrapper {
        height: auto;
        margin: 1 0;
        border: round #58a6ff;
        border-title-align: left;
        padding: 1 2;
        background: #0d1624;
    }

    EndpointWrapper .label {
        color: #58a6ff;
        text-style: bold italic;
    }

    EndpointWrapper .value {
        color: #cdd9e5;
        text-style: underline;
    }
    """

    def __init__(self, value: str, **kwargs):
        super().__init__(label="Endpoint: ", value=value, **kwargs)



from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.widgets import Header, Label

from model.cli.command.http.HttpOutputModel import HttpOutput
from view.CommandWidget import CommandWidget
from wrapper.output.http.BodyWrapper import BodyWrapper
from wrapper.output.http.EndpointWrapper import EndpointWrapper
from wrapper.output.http.HeadersWrapper import HeadersWrapper
from wrapper.output.http.StatusCodeWrapper import StatusCodeWrapper


class HttpCommandWrapper(CommandWidget):
    DEFAULT_CSS = """
    HttpCommandWrapper {
        height: auto;
        background: #0d1117;
    }

    .left-panel {
        width: 50%;
        padding: 0 1;
        border-right: solid #21262d;
        align: left top;
    }

    .right-panel {
        width: 50%;
        padding: 0 1;
        align: left top;
    }

    .right-panel Collapsible {
        margin: 1 0;
        padding: 0;
        background: #161b22;
        border: round #30363d;
        height: auto;
    }

    .right-panel CollapsibleTitle {
        color: #6e7681;
        text-style: italic;
        background: #0d1117;
        padding: 0 2;
        margin: 0;
    }

    .right-panel CollapsibleTitle:hover {
        color: #58a6ff;
        background: #161b22;
    }

    .right-panel CollapsibleContent {
        height: auto;
        max-height: 24;
        padding: 0;
        margin: 0;
        background: #161b22;
    }

    .right-panel ScrollableContainer {
        border: round #30363d;
        background: #161b22;
        padding: 1 2;
        height: 24;
        scrollbar-color: #30363d;
        scrollbar-size-vertical: 1;
    }
    .panel-label {
        color: #6e7681;
        text-style: bold;
        margin: 0 0 1 0;
        padding: 0 1;
        border-bottom: solid #21262d;
        width: 100%;
    }
    """

    _my_headers: HeadersWrapper
    _my_status_code: StatusCodeWrapper
    _my_body: BodyWrapper
    _my_endpoint: EndpointWrapper

    def __init__(self, http_output: HttpOutput, **kwargs):
        super().__init__(**kwargs)
        self._my_headers = HeadersWrapper(
            http_output.get_output().get_headers())
        self._my_body = BodyWrapper(
            http_output.get_output().get_body().get_content())
        self._my_endpoint = EndpointWrapper(
            http_output.get_output().get_endpoint().dump())
        self._my_status_code = StatusCodeWrapper(
            http_output.get_output().get_status_code())

    def compose(self) -> ComposeResult:
        with Header():
            yield Label(content="Http command")
        with Horizontal():
            with Vertical(classes="left-panel"):
                yield Label("OVERVIEW", classes="panel-label")
                yield self._my_endpoint
                yield self._my_status_code
            with Vertical(classes="right-panel"):
                yield Label("PAYLOAD", classes="panel-label")
                yield self._my_body
                yield self._my_headers



from model.cli.command.FlagFactory import AbstractFlagFactory
from model.cli.command.http.Contants import HTTP_FLAG_METHOD, HTTP_FLAG_ENDPOINT, HTTP_FLAG_HEADERS, \
    HTTP_FLAG_PARAMETERS
from model.cli.command.http.flags.HttpEndpointFlagModel import HttpEndpointFlag, HttpEndpointFlagValue
from model.cli.command.http.flags.HttpHeadersFlagModel import HttpHeadersFlag, HttpHeadersFlagValue
from model.cli.command.http.flags.HttpMethodFlagModel import HttpMethodFlag, HttpMethodFlagValue
from model.cli.command.http.flags.HttpParametersFlagModel import HttpParametersFlag, HttpParametersFlagValue


class HttpFlagFactory(AbstractFlagFactory):
    _my_flags = {
        HTTP_FLAG_METHOD: (HttpMethodFlag, HttpMethodFlagValue),
        HTTP_FLAG_ENDPOINT: (HttpEndpointFlag, HttpEndpointFlagValue),
        HTTP_FLAG_HEADERS: (HttpHeadersFlag, HttpHeadersFlagValue),
        HTTP_FLAG_PARAMETERS: (HttpParametersFlag, HttpParametersFlagValue),
    }



from model.cli.command.OutputModel import AbstractOutput
from model.cli.command.http.HttpOutputModel import HttpOutput
from view.CommandWidget import CommandWidget
from wrapper.output.http.HttpCommandWrapper import HttpCommandWrapper


def get_viewer_command_widget[T](output: AbstractOutput[T]) -> CommandWidget:
    if isinstance(output, HttpOutput):
        return HttpCommandWrapper(output)
    raise NotImplementedError("The output format is not yet supported")
